@@ -13,6 +13,7 @@ class BaseDelegate(object):
         self.targets = {}
         self.owner = owner
         self.linker = linker
+        self.get_id = self.linker.get_id
 
     def __len__(self):
         return len(self.targets)
@@ -49,7 +50,7 @@ class DelegateSet(BaseDelegate, Set):
         return iter(self.targets.values())
 
     def __contains__(self, target):
-        return id(target) in self.targets
+        return self.get_id(target) in self.targets
 
     def __getitem__(self, index):
         """ slow way to getting access to specific targets """
@@ -59,10 +60,10 @@ class DelegateSet(BaseDelegate, Set):
         raise IndexError(str(index))
 
     def _set(self, target):
-        self.targets[id(target)] = target
+        self.targets[self.get_id(target)] = target
 
     def _del(self, target):
-        self.targets.pop(id(target), None)
+        self.targets.pop(self.get_id(target), None)
 
     def __str__(self):
         return f"{{{', '.join(map(str, self))}}}"
@@ -146,7 +147,7 @@ class BaseLinker(object):
         self._do_on_link = on_link
         self._do_on_unlink = on_unlink
         self._install = install
-        self._get_id = get_id or self.get_id
+        self.get_id = get_id or self.get_id
         self.name = None
 
     @property
@@ -187,7 +188,7 @@ class BaseLinker(object):
         except KeyError:
             return self._init(obj)  # initializes attribute on first access
 
-    def iterate(self, start_obj, cyclic=False, breadth_first=False, stop=lambda obj: False):
+    def iterate(self, start_obj, cyclic=False, breadth_first=False):
         """
         iterate through the graph
         :param start_obj: starting object from which the graph is followed
@@ -195,7 +196,7 @@ class BaseLinker(object):
         :param breadth_first: depth_first iteration if false (default) else breadth_first iteration
         :yield: nodes in the graph
         """
-        yield from Iterator(self.name).iterate(start_obj, cyclic=cyclic, breadth_first=breadth_first, stop=stop)
+        yield from Iterator(self.name).iterate(start_obj, cyclic=cyclic, breadth_first=breadth_first)
 
     __call__ = iterate  # shortcut to iteration
 
@@ -220,7 +221,7 @@ class BaseLinker(object):
         """
         Gather all nodes in a graph, going forward and backward if reverse_name is defined.
         """
-        get_id = self._get_id
+        get_id = self.get_id
 
         forw_iterator = Iterator(self.name)
         back_iterator = Iterator(self.reverse_name) if self.reverse_name else None
@@ -246,7 +247,7 @@ class BaseLinker(object):
         """
         Gather all connected pairs of nodes in a graph, going forward, and backward if reverse_name is defined.
         """
-        get_id = self._get_id
+        get_id = self.get_id
 
         forw_iterator = Iterator(self.name)
         back_iterator = Iterator(self.reverse_name) if self.reverse_name else None
@@ -312,7 +313,7 @@ class BaseLinker(object):
 
         def visit(obj):
             nonlocal cyclic
-            obj_id = self._get_id(obj)
+            obj_id = self.get_id(obj)
             if obj_id in seen:
                 cyclic = True
                 raise StopIteration
@@ -330,17 +331,17 @@ class BaseLinker(object):
          Finds the shortest path through the graph from start_obj to target_obj
         :param start_obj: node to start from
         :param target_obj: node to which the path must be calculated
-        :param get_cost(node, next_node): cost function: must return cost for edge between node and next_node. Default
+        :param get_cost(node, next_node): cost function: must return cost for following edge between node and next_node. Default
             results in a shortest path defined by the number of edges between start and end.
-        :param heuristic(node, target_node): optional heuristic function to calculate an lower estimate of the remaining
-            cost from a node to the target node (using A* algorithm, often resulting in faster path_finding).
+        :param heuristic(node, target_node): optional heuristic function to calculate an under estimate of the remaining
+            cost from a node to the target node (often resulting in faster path_finding, using A*).
         :return: list of nodes of the shortest path
         """
         return Iterator(self.name).shortest_path(start_obj, target_obj,
                                                  get_cost=get_cost,
                                                  heuristic=heuristic)
 
-    def save_image(self, start_obj, filename, label_getter=lambda obj: str,
+    def save_image(self, start_obj, filename, label_getter=lambda obj: obj.name,
                    view=False, fontsize='10', fontname='Arial bold', **options):
 
         if self.is_directed:
@@ -399,15 +400,15 @@ class BaseLinker(object):
 
     def _on_link(self, obj, target, _remote=False):
         if self._do_on_link:
-            self._do_on_link(obj, target)
+            return self._do_on_link(obj, target)
         if self.reverse_name and not _remote:
-            self._reverse(target)._on_link(target, obj, True)
+            return self._reverse(target)._on_link(target, obj, True)
 
     def _on_unlink(self, obj, target, _remote=False):
         if self._do_on_unlink:
-            self._do_on_unlink(obj, target)
+            return self._do_on_unlink(obj, target)
         if self.reverse_name and not _remote:
-            self._reverse(target)._on_unlink(target, obj, True)
+            return self._reverse(target)._on_unlink(target, obj, True)
 
     def _init(self, obj):
         raise NotImplementedError
@@ -445,7 +446,7 @@ class One(BaseLinker):
         return target is not None and self.__get__(obj) is target
 
     def _build_on_visit(self, key, _reg):
-        get_id = self._get_id
+        get_id = self.get_id
 
         def visit(obj):
             if isinstance(key, str):
@@ -488,7 +489,7 @@ class BaseMany(BaseLinker):
 
     def _build_on_visit(self, key, _reg):
         name = self.name
-        get_id = self._get_id
+        get_id = self.get_id
 
         def visit(obj, key=key):
             if isinstance(key, str):
